@@ -79,6 +79,7 @@ static functor_t FUNCTOR_at1;
 static functor_t FUNCTOR_eval1;
 
 static int py_initialize_done = FALSE;
+static int py_module_initialize_done = FALSE;
 static int py_finalizing = FALSE;
 static int py_gil_thread = 0;	/* Prolog thread that owns the GIL */
 
@@ -391,32 +392,55 @@ get_py_name(term_t name, PyObject **obj, int flags)
 }
 
 static htable *py_module_table = NULL;
+static PL_option_t import_options[] =
+{ PL_OPTION("as", OPT_ATOM),
+  PL_OPTIONS_END
+};
+
 
 static int
-py_register_module(term_t name, PyObject **mod, int flags)
-{ PyObject *idobj;
-  atom_t id;
+py_register_module(term_t name, term_t options, PyObject **mod, int flags)
+{ PyObject *idobj = NULL;
+  int rc = FALSE;
 
   if ( !py_module_table )
     py_module_table = py_new_hashmap();
 
-  if ( get_py_name(name, &idobj, flags) &&
-       PL_get_atom(name, &id) )
-  { PyObject *m = check_error(PyImport_Import(idobj));
+  if ( get_py_name(name, &idobj, flags) )
+  { PyObject *m;
+    atom_t as = 0;
 
+    if ( options )
+    { if ( !PL_scan_options(options, 0, "py_import_options", import_options,
+			    &as) )
+	goto out;
+      if ( as && py_lookup_hashmap(py_module_table, as) )
+      { term_t as_term;
+	rc = ((as_term=PL_new_term_ref()) &&
+	      PL_put_atom(as_term, as) &&
+	      PL_permission_error("import_as", "py_module", as_term));
+	goto out;
+      }
+    }
+    if ( !as && !PL_get_atom_ex(name, &as) )
+      goto out;
+
+    m = check_error(PyImport_Import(idobj));
     Py_DECREF(idobj);
     if ( m )
     { PyObject *old = NULL;
       if ( mod )
 	*mod = m;
-      int rc = py_add_hashmap(py_module_table, id, m, &old);
+      rc = py_add_hashmap(py_module_table, as, m, &old);
       if ( old )
 	Py_DECREF(old);
       return rc;
     }
   }
 
-  return FALSE;
+out:
+  Py_CLEAR(idobj);
+  return rc;
 }
 
 
@@ -442,7 +466,7 @@ get_py_module(term_t name, PyObject **mod)
       *mod = obj;
       return TRUE;
     } else
-    { if ( py_register_module(name, mod, 0) )
+    { if ( py_register_module(name, 0, mod, 0) )
       { Py_INCREF(*mod);
 	return TRUE;
       }
@@ -460,7 +484,21 @@ py_update_module_cache(term_t name)
 
   if ( !py_gil_ensure(&state) )
     return FALSE;
-  rc = py_register_module(name, NULL, CVT_EXCEPTION);
+  rc = py_register_module(name, 0, NULL, CVT_EXCEPTION);
+  py_gil_release(state);
+
+  return rc;
+}
+
+
+static foreign_t
+py_import(term_t spec, term_t options)
+{ py_gil_state state;
+  int rc;
+
+  if ( !py_gil_ensure(&state) )
+    return FALSE;
+  rc = py_register_module(spec, options, NULL, CVT_EXCEPTION);
   py_gil_release(state);
 
   return rc;
@@ -751,39 +789,44 @@ out:
 
 static int
 py_unify_portable_dict(term_t t, PyObject *obj, int flags)
-{ Py_ssize_t size = PyDict_Size(obj);
-  term_t pl_kv, pl_av, tail, tmp;
-  PyObject *py_key, *py_value;
+{ PyObject *py_key, *py_value;
   Py_ssize_t i = 0;
 
-  if ( size == 0 )
-    return PL_unify_term(t, PL_FUNCTOR, FUNCTOR_py1, PL_ATOM, ATOM_curl);
+  int rc = PyDict_Next(obj, &i, &py_key, &py_value);
+  if ( rc )
+  { term_t pl_kv, pl_av, tail, tmp;
 
-  if ( !(pl_kv=PL_new_term_ref()) ||
-       !(tail=PL_new_term_ref()) ||
-       !(pl_av=PL_new_term_refs(2)) )
-    return FALSE;
-  tmp = pl_av+0;
+    if ( !(pl_kv=PL_new_term_ref()) ||
+	 !(tail=PL_new_term_ref()) ||
+	 !(pl_av=PL_new_term_refs(2)) )
+      return FALSE;
+    tmp = pl_av+0;
 
-  if ( !PL_unify_term(t, PL_FUNCTOR, FUNCTOR_curl1, PL_TERM, tail) )
-    return FALSE;
+    if ( !PL_unify_term(t, PL_FUNCTOR, FUNCTOR_curl1, PL_TERM, tail) )
+      return FALSE;
 
-  for( Py_ssize_t pli=1; PyDict_Next(obj, &i, &py_key, &py_value); pli++ )
-  { if ( !PL_put_variable(pl_av+0) ||
-	 !PL_put_variable(pl_av+1) ||
-	 !py_unify(pl_av+0, py_key, flags) ||
-	 !py_unify(pl_av+1, py_value, flags) ||
-	 !PL_cons_functor_v(pl_kv, FUNCTOR_key_value2, pl_av) )
-      return FALSE;		/* pl_kv is now Key:Value */
-    if ( pli < size )
-    { if ( !PL_put_variable(tmp) ||
-	   !PL_unify_term(tail, PL_FUNCTOR, FUNCTOR_comma2,
-			  PL_TERM, pl_kv, PL_TERM, tmp) ||
-	   !PL_put_term(tail, tmp) )
-	return FALSE;
-    } else
-    { return PL_unify(tail, pl_kv);
+    while( rc  )
+    { if ( !PL_put_variable(pl_av+0) ||
+	   !PL_put_variable(pl_av+1) ||
+	   !py_unify(pl_av+0, py_key, flags) ||
+	   !py_unify(pl_av+1, py_value, flags) ||
+	   !PL_cons_functor_v(pl_kv, FUNCTOR_key_value2, pl_av) )
+	return FALSE;		/* pl_kv is now Key:Value */
+
+      rc = PyDict_Next(obj, &i, &py_key, &py_value);
+
+      if ( rc )
+      { if ( !PL_put_variable(tmp) ||
+	     !PL_unify_term(tail, PL_FUNCTOR, FUNCTOR_comma2,
+			    PL_TERM, pl_kv, PL_TERM, tmp) ||
+	     !PL_put_term(tail, tmp) )
+	  return FALSE;
+      } else
+      { return PL_unify(tail, pl_kv);
+      }
     }
+  } else
+  { return PL_unify_term(t, PL_FUNCTOR, FUNCTOR_py1, PL_ATOM, ATOM_curl);
   }
 
   assert(0);
@@ -791,6 +834,22 @@ py_unify_portable_dict(term_t t, PyObject *obj, int flags)
 }
 
 
+#define DICT_FAST_KEYS 25
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Unify a Python dict.  If possible we do this as a SWI-Prolog dict.  If
+there are  invalid keys we  call py_unify_portable_dict() to  create a
+{k:v, ...} list.
+
+(*) It  turns out that PyDict_Next()  may return more elements  in the
+dict than PyDict_Size()  told us to be there.  Why?   Could it be that
+the functions  we call indirectly  extend the  dict?  In any  case, we
+cannot trust size and we must  extend the data structures to deal with
+the additional keys.   The test case is
+
+    ?- py_call(sys:modules, M).
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 static int
 py_unify_dict(term_t t, PyObject *obj, int flags, fid_t fid)
 { if ( (flags&PYU_CURL) )
@@ -799,31 +858,60 @@ py_unify_dict(term_t t, PyObject *obj, int flags, fid_t fid)
   Py_ssize_t size = PyDict_Size(obj);
   term_t pl_dict = PL_new_term_ref();
   term_t pl_values;
-  atom_t fast[25];
+  atom_t fast[DICT_FAST_KEYS];
   atom_t *pl_keys;
   int rc = FALSE;
   Py_ssize_t i = 0;
+  Py_ssize_t pli = 0;
   PyObject *py_key, *py_value;
   int invalid_keys = FALSE;
 
-  if ( size < 0 || size > INT_MAX )
-    return PL_resource_error("stack");
+  if ( !(pl_values = PL_new_term_refs((size_t)size)) )
+    return FALSE;
 
-  pl_values = PL_new_term_refs((int)size);
-
-  if ( size < 25 )
-    pl_keys = fast;
-  else if ( !(pl_keys = malloc(size*sizeof(atom_t))) )
-    return PL_resource_error("memory");
-  memset(pl_keys, 0, size*sizeof(atom_t));
-
-  if ( !pl_keys )
+  if ( size <= DICT_FAST_KEYS )
+  { pl_keys = fast;
+  } else if ( !(pl_keys = malloc(size*sizeof(atom_t))) )
   { PL_resource_error("memory");
     goto out;
   }
+  memset(pl_keys, 0, size*sizeof(atom_t));
 
-  for( size_t pli=0; PyDict_Next(obj, &i, &py_key, &py_value); pli++ )
-  { if ( PyUnicode_Check(py_key) )
+  for( ; PyDict_Next(obj, &i, &py_key, &py_value); pli++ )
+  { if ( pli == size )		/* see (*) */
+    { size_t ext = size/2;
+      if ( ext == 0 )
+	ext = 1;
+
+      term_t nv = PL_new_term_refs(ext); /* This should give us the next chunk */
+      if ( !nv )			 /* if all py_unify() properly reset */
+	goto out;
+      assert(nv == pl_values+size);
+
+      if ( size+ext > DICT_FAST_KEYS )
+      { if ( pl_keys == fast )
+	{ atom_t *nkeys = malloc((size+ext)*sizeof(atom_t));
+	  if ( nkeys )
+	  { memcpy(nkeys, pl_keys, sizeof(atom_t)*size);
+	    pl_keys = nkeys;
+	  } else
+	  { PL_resource_error("memory");
+	    goto out;
+	  }
+	} else
+	{ atom_t *nkeys = realloc(pl_keys, (size+ext)*sizeof(atom_t));
+	  if ( nkeys )
+	  { pl_keys = nkeys;
+	  } else
+	  { PL_resource_error("memory");
+	    goto out;
+	  }
+	}
+      }
+      size += ext;
+    }
+
+    if ( PyUnicode_Check(py_key) )
     { ssize_t len;
       wchar_t *s;
 
@@ -845,11 +933,11 @@ py_unify_dict(term_t t, PyObject *obj, int flags, fid_t fid)
       goto out;
   }
 
-  rc = (PL_put_dict(pl_dict, ATOM_pydict, size, pl_keys, pl_values) &&
+  rc = (PL_put_dict(pl_dict, ATOM_pydict, pli, pl_keys, pl_values) &&
 	PL_unify(t, pl_dict));
 
 out:
-  _PL_unregister_keys(size, pl_keys);
+  _PL_unregister_keys(pli, pl_keys);
   if ( pl_keys != fast )
     free(pl_keys);
 
@@ -2423,6 +2511,7 @@ install_janus(void)
 
   REGISTER("py_initialize_",	     3,	py_initialize_,		0);
   REGISTER("py_finalize",	     0,	py_finalize,		0);
+  REGISTER("py_import_",	     2, py_import,		0);
   REGISTER("py_call",		     1,	py_call1,		0);
   REGISTER("py_call",		     2,	py_call2,		0);
   REGISTER("py_call",		     3,	py_call3,		0);
@@ -2438,7 +2527,8 @@ install_janus(void)
   REGISTER("py_debug",		     1,	py_debug,		0);
   REGISTER("py_update_module_cache", 1,	py_update_module_cache,	0);
 
-  if ( PyImport_AppendInittab("_swipl", PyInit__swipl) == -1 )
+  if ( !py_module_initialize_done &&
+       PyImport_AppendInittab("_swipl", PyInit__swipl) == -1 )
     Sdprintf("Failed to add module swipl to Python");
 }
 
