@@ -130,7 +130,8 @@ be kept consistent.
 #define TAG_VAR		0x00000000L	/* tag for variable (= 0L) */
 #define TAG_ATTVAR	0x00000001L	/* tag for attributed variable */
 #define TAG_FLOAT	0x00000002L	/* Floating point number */
-#define TAG_INTEGER	0x00000003L	/* Tagged or indirect integer */
+#define TAG_INTEGER	0x00000003L	/* Tagged small int, 64-bit indirect, */
+					/* GMP mpz or mpq (rational) */
 #define TAG_STRING	0x00000004L	/* String */
 #define TAG_ATOM	0x00000005L	/* an atom */
 #define TAG_COMPOUND	0x00000006L	/* Compound term */
@@ -214,7 +215,7 @@ and while loading .wic files.  It comes at no price.
 #define isTerm(w)	(tag(w)   == TAG_COMPOUND)
 #define isConst(w)	(isAtom(w) || isTaggedInt(w)) /* H_ATOM, B_ATOM, H_SMALLINT, B_SMALLINT */
 
-#ifdef O_GMP
+#if O_BIGNUM
 #define isInteger(w)	( isTaggedInt(w) || \
 			  ((tag(w) == TAG_INTEGER) && !isMPQNum(w)) )
 #else
@@ -296,20 +297,8 @@ and while loading .wic files.  It comes at no price.
 			 wsizeofIndirect(w) == sizeof(int64_t)/sizeof(word))
 
 #define MP_RAT_MASK	(0x1)
-#define isMPQNum(w)	isMPQNum__LD(w PASS_LD)
-#define isMPZNum(w)	isMPZNum__LD(w PASS_LD)
-
-#if ALIGNOF_INT64_T == ALIGNOF_VOIDP
-#define valBignum(w)	(*(int64_t *)valIndirectP(w))
-#else
-#define valBignum(w)	valBignum__LD(w PASS_LD)
-#endif
-#if ALIGNOF_DOUBLE == ALIGNOF_VOIDP
-#define valFloat(w)	(*(double *)valIndirectP(w))
-#else
-#define valFloat(w)	valFloat__LD(w PASS_LD)
-#endif
 
+/* valBignum(w) and valFloat(w) moved to pl-inline.h */
 #define isBString(w)	(isString(w) && ((char *)valIndirectP(w))[0] == 'B')
 #define isWString(w)	(isString(w) && ((char *)valIndirectP(w))[0] == 'W')
 
@@ -360,5 +349,3 @@ static inline Atom	fetchAtomArray(size_t index);
 
 #define consInt(n)	(((word)(n)<<LMASK_BITS) | TAG_INTEGER)
 #define consUInt(n)	(((word)(n)<<LMASK_BITS) | TAG_INTEGER)
-
-
